#!/bin/bash

#------ SET FOR YOUR SYSTEM -------
#
GPU_COUNT=4
CPU_COUNT=16
#Should we run large STMV 1 million atom + test?
RUN_LARGE_BENCH=true
#
#----------------------------------

#This benchmark script runs AMBER 18 benchmarks on intranode CPUs and intranode GPUs.
#It assumes AMBERHOME is set correctly.

function bench_cpu {
  if [ $CPU_COUNT -gt 0 ]; then
    echo -n "CPU code $CPU_COUNT cores: "
    mpirun -np $CPU_COUNT $AMBERHOME/bin/pmemd.MPI -O -i mdin.CPU -o mdout.$CPU_COUNT -inf mdinfo.$CPU_COUNT -x mdcrd.$CPU_COUNT -r restrt.$CPU_COUNT
    grep "ns/day" mdinfo.$CPU_COUNT | tail -n1
  fi
}

function bench_gpu_single {
  if [ $GPU_COUNT -gt 0 ]; then
    echo -n "      [0] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=0
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
}

function bench_gpu_single_all_seq {
  if [ $GPU_COUNT -gt 0 ]; then
    echo -n "      [0] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=0
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 1 ]; then
    echo -n "      [1] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=1
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 2 ]; then
    echo -n "      [2] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=2
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 3 ]; then
    echo -n "      [3] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=3
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 4 ]; then
    echo -n "      [4] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=4
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 5 ]; then
    echo -n "      [5] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=5
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 6 ]; then
    echo -n "      [6] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=6
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
  if [ $GPU_COUNT -gt 7 ]; then
    echo -n "      [7] 1 x GPU: "
    export CUDA_VISIBLE_DEVICES=7
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU -inf mdinfo.1GPU -x mdcrd.1GPU -r restrt.1GPU
    grep "ns/day" mdinfo.1GPU | tail -n1
  fi
}

function bench_gpu_single_all {
  if [ $GPU_COUNT -gt 0 ]; then
    export CUDA_VISIBLE_DEVICES=0
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 1 ]; then
    export CUDA_VISIBLE_DEVICES=1
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 2 ]; then
    export CUDA_VISIBLE_DEVICES=2
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 3 ]; then
    export CUDA_VISIBLE_DEVICES=3
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 4 ]; then
    export CUDA_VISIBLE_DEVICES=4
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 5 ]; then
    export CUDA_VISIBLE_DEVICES=5
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 6 ]; then
    export CUDA_VISIBLE_DEVICES=6
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi
  if [ $GPU_COUNT -gt 7 ]; then
    export CUDA_VISIBLE_DEVICES=7
    $AMBERHOME/bin/pmemd.cuda -O -i mdin.GPU -o mdout.1GPU_$CUDA_VISIBLE_DEVICES -inf mdinfo.1GPU_$CUDA_VISIBLE_DEVICES -x mdcrd.1GPU_$CUDA_VISIBLE_DEVICES -r restrt.1GPU_$CUDA_VISIBLE_DEVICES &
  fi

  wait

  echo "Multiple Single GPU Run Performance"
  if [ $GPU_COUNT -gt 0 ]; then
    export CUDA_VISIBLE_DEVICES=0
    echo -n "      [0] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi 
  if [ $GPU_COUNT -gt 1 ]; then
    export CUDA_VISIBLE_DEVICES=1
    echo -n "      [1] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi
  if [ $GPU_COUNT -gt 2 ]; then
    export CUDA_VISIBLE_DEVICES=2
    echo -n "      [2] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi
  if [ $GPU_COUNT -gt 3 ]; then
    export CUDA_VISIBLE_DEVICES=3
    echo -n "      [3] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi
  if [ $GPU_COUNT -gt 4 ]; then
    export CUDA_VISIBLE_DEVICES=4
    echo -n "      [4] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi
  if [ $GPU_COUNT -gt 5 ]; then
    export CUDA_VISIBLE_DEVICES=5
    echo -n "      [5] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi
  if [ $GPU_COUNT -gt 6 ]; then
    export CUDA_VISIBLE_DEVICES=6
    echo -n "      [6] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi
  if [ $GPU_COUNT -gt 7 ]; then
    export CUDA_VISIBLE_DEVICES=7
    echo -n "      [7] 1 x GPU: "
    grep "ns/day" mdinfo.1GPU_$CUDA_VISIBLE_DEVICES | tail -n1
  fi

}

function bench_2gpus_multi {
  if [ $GPU_COUNT -gt 1 ]; then
    export CUDA_VISIBLE_DEVICES=0,1
    mpirun -np 2 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.2GPU_0 -inf mdinfo.2GPU_0 -x mdcrd.2GPU_0 -r restrt.2GPU_0 &
  fi
  if [ $GPU_COUNT -gt 3 ]; then
    export CUDA_VISIBLE_DEVICES=2,3
    mpirun -np 2 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.2GPU_2 -inf mdinfo.2GPU_2 -x mdcrd.2GPU_2 -r restrt.2GPU_2 &
  fi
  if [ $GPU_COUNT -gt 5 ]; then
    export CUDA_VISIBLE_DEVICES=4,5
    mpirun -np 2 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.2GPU_4 -inf mdinfo.2GPU_4 -x mdcrd.2GPU_4 -r restrt.2GPU_4 &
  fi
  if [ $GPU_COUNT -gt 7 ]; then
    export CUDA_VISIBLE_DEVICES=6,7
    mpirun -np 2 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.2GPU_6 -inf mdinfo.2GPU_6 -x mdcrd.2GPU_6 -r restrt.2GPU_6 &
  fi

  wait

  echo "Multiple 2xGPU Run Performance"
  if [ $GPU_COUNT -gt 1 ]; then
    echo -n "      [0,1] 2 x GPU: "
    grep "ns/day" mdinfo.2GPU_0 | tail -n1
  fi 
  if [ $GPU_COUNT -gt 3 ]; then
    echo -n "      [2,3] 2 x GPU: "
    grep "ns/day" mdinfo.2GPU_2 | tail -n1
  fi 
  if [ $GPU_COUNT -gt 5 ]; then
    echo -n "      [4,5] 2 x GPU: "
    grep "ns/day" mdinfo.2GPU_4 | tail -n1
  fi 
  if [ $GPU_COUNT -gt 7 ]; then
    echo -n "      [6,7] 2 x GPU: "
    grep "ns/day" mdinfo.2GPU_6 | tail -n1
  fi 

}

function bench_gpus {
  if [ $GPU_COUNT -gt 1 ]; then
    export CUDA_VISIBLE_DEVICES=0,1
    mpirun -np 2 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.2GPU -inf mdinfo.2GPU -x mdcrd.2GPU -r restrt.2GPU
    grep -q "Peer to Peer support: ENABLED" mdout.2GPU
    if [ $? = 0 ]; then
      echo -n " P2P"
    else
      echo -n "    " 
    fi
    echo -n "      2 x GPU: "
    grep "ns/day" mdinfo.2GPU | tail -n1
  fi

  if [ $GPU_COUNT -gt 3 ]; then
    export CUDA_VISIBLE_DEVICES=0,1,2,3
    mpirun -np 4 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.4GPU -inf mdinfo.4GPU -x mdcrd.4GPU -r restrt.4GPU
    grep -q "Peer to Peer support: ENABLED" mdout.4GPU
    if [ $? = 0 ]; then
      echo -n " P2P"
    else
      echo -n "    " 
    fi
    echo -n "      4 x GPU: "
    grep "ns/day" mdinfo.4GPU | tail -n1
  fi

  if [ $GPU_COUNT -gt 7 ]; then
    export CUDA_VISIBLE_DEVICES=0,1,2,3,4,5,6,7
    mpirun -np 8 $AMBERHOME/bin/pmemd.cuda.MPI -O -i mdin.GPU -o mdout.8GPU -inf mdinfo.8GPU -x mdcrd.8GPU -r restrt.8GPU
    grep -q "Peer to Peer support: ENABLED" mdout.8GPU
    if [ $? = 0 ]; then
      echo -n " P2P"
    else
      echo -n "    " 
    fi
    echo -n "      8 x GPU: "
    grep "ns/day" mdinfo.8GPU | tail -n1
  fi

}


echo ""
echo "JAC_PRODUCTION_NVE - 23,558 atoms PME 4fs"
echo "-----------------------------------------"
echo ""
cd PME/JAC_production_NVE_4fs

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "JAC_PRODUCTION_NPT - 23,558 atoms PME 4fs"
echo "-----------------------------------------"
echo ""
cd ../JAC_production_NPT_4fs

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "JAC_PRODUCTION_NVE - 23,558 atoms PME 2fs"
echo "-----------------------------------------"
echo ""
cd ../JAC_production_NVE

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "JAC_PRODUCTION_NPT - 23,558 atoms PME 2fs"
echo "-----------------------------------------"
echo ""
cd ../JAC_production_NPT

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "FACTOR_IX_PRODUCTION_NVE - 90,906 atoms PME"
echo "-------------------------------------------"
echo ""
cd ../FactorIX_production_NVE

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "FACTOR_IX_PRODUCTION_NPT - 90,906 atoms PME"
echo "-------------------------------------------"
echo ""
cd ../FactorIX_production_NPT

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "CELLULOSE_PRODUCTION_NVE - 408,609 atoms PME"
echo "--------------------------------------------"
echo ""
cd ../Cellulose_production_NVE

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "CELLULOSE_PRODUCTION_NPT - 408,609 atoms PME"
echo "--------------------------------------------"
echo ""
cd ../Cellulose_production_NPT

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

if [ "$RUN_LARGE_BENCH" = true ]; then
echo ""
echo "STMV_PRODUCTION_NPT - 1,067,095 atoms PME"
echo "-----------------------------------------"
echo ""
cd ../STMV_production_NPT_4fs

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10
fi

echo ""
echo "TRPCAGE_PRODUCTION - 304 atoms GB"
echo "---------------------------------"
echo ""
cd ../../GB/TRPCage

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10

echo ""
echo "MYOGLOBIN_PRODUCTION - 2,492 atoms GB"
echo "-------------------------------------"
echo ""
cd ../myoglobin

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10

echo ""
echo "NUCLEOSOME_PRODUCTION - 25,095 atoms GB"
echo "---------------------------------------"
echo ""
cd ../nucleosome

bench_cpu
sleep 10
bench_gpu_single_all_seq
sleep 10
bench_gpu_single_all
sleep 10
bench_gpus
sleep 10
bench_2gpus_multi
sleep 10


